@@ -18,15 +18,13 @@
 #include "mozilla/dom/SVGSVGElement.h"
 #include "mozilla/dom/SVGDocument.h"
 #include "mozilla/gfx/2D.h"
-#include "mozilla/gfx/gfxVars.h"
-#include "mozilla/layers/LayerManager.h"
 #include "mozilla/PendingAnimationTracker.h"
 #include "mozilla/PresShell.h"
 #include "mozilla/ProfilerLabels.h"
 #include "mozilla/RefPtr.h"
 #include "mozilla/StaticPrefs_image.h"
 #include "mozilla/SVGObserverUtils.h"  // for SVGRenderingObserver
-#include "mozilla/Tuple.h"
+
 #include "nsIStreamListener.h"
 #include "nsMimeTypes.h"
 #include "nsPresContext.h"
@@ -38,15 +36,16 @@
 #include "ISurfaceProvider.h"
 #include "LookupResult.h"
 #include "Orientation.h"
-#include "SourceSurfaceBlobImage.h"
 #include "SVGDocumentWrapper.h"
 #include "SVGDrawingCallback.h"
 #include "SVGDrawingParameters.h"
 #include "nsIDOMEventListener.h"
 #include "SurfaceCache.h"
+#include "BlobSurfaceProvider.h"
 #include "mozilla/dom/Document.h"
 #include "mozilla/dom/DocumentInlines.h"
 #include "mozilla/image/Resolution.h"
+#include "WindowRenderer.h"
 
 namespace mozilla {
 
@@ -264,9 +263,7 @@ bool SVGDrawingCallback::operator()(gfxContext* aContext,
   gfxContextAutoSaveRestore contextRestorer(aContext);
 
   // Clip to aFillRect so that we don't paint outside.
-  aContext->NewPath();
-  aContext->Rectangle(aFillRect);
-  aContext->Clip();
+  aContext->Clip(aFillRect);
 
   gfxMatrix matrix = aTransform;
   if (!matrix.Invert()) {
@@ -372,15 +369,7 @@ size_t VectorImage::SizeOfSourceWithComputedFallback(
   return windowSizes.getTotalSize();
 }
 
-void VectorImage::CollectSizeOfSurfaces(
-    nsTArray<SurfaceMemoryCounter>& aCounters,
-    MallocSizeOf aMallocSizeOf) const {
-  SurfaceCache::CollectSizeOfSurfaces(ImageKey(this), aCounters, aMallocSizeOf);
-  ImageResource::CollectSizeOfSurfaces(aCounters, aMallocSizeOf);
-}
-
 nsresult VectorImage::OnImageDataComplete(nsIRequest* aRequest,
-                                          nsISupports* aContext,
                                           nsresult aStatus, bool aLastPart) {
   // Call our internal OnStopRequest method, which only talks to our embedded
   // SVG document. This won't have any effect on our ProgressTracker.
@@ -406,7 +395,6 @@ nsresult VectorImage::OnImageDataComplete(nsIRequest* aRequest,
 }
 
 nsresult VectorImage::OnImageDataAvailable(nsIRequest* aRequest,
-                                           nsISupports* aContext,
                                            nsIInputStream* aInStr,
                                            uint64_t aSourceOffset,
                                            uint32_t aCount) {
@@ -476,12 +464,12 @@ VectorImage::GetWidth(int32_t* aWidth) {
 }
 
 //******************************************************************************
-nsresult VectorImage::GetNativeSizes(nsTArray<IntSize>& aNativeSizes) const {
+nsresult VectorImage::GetNativeSizes(nsTArray<IntSize>& aNativeSizes) {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 //******************************************************************************
-size_t VectorImage::GetNativeSizesLength() const { return 0; }
+size_t VectorImage::GetNativeSizesLength() { return 0; }
 
 //******************************************************************************
 NS_IMETHODIMP_(void)
@@ -490,8 +478,13 @@ VectorImage::RequestRefresh(const TimeStamp& aTime) {
     return;
   }
 
-  PendingAnimationTracker* tracker =
-      mSVGDocumentWrapper->GetDocument()->GetPendingAnimationTracker();
+  Document* doc = mSVGDocumentWrapper->GetDocument();
+  if (!doc) {
+    // We are racing between shutdown and a refresh.
+    return;
+  }
+
+  PendingAnimationTracker* tracker = doc->GetPendingAnimationTracker();
   if (tracker && ShouldAnimate()) {
     tracker->TriggerPendingAnimationsOnNextTick(aTime);
   }
@@ -522,12 +515,10 @@ void VectorImage::SendInvalidationNotifications() {
   // notifications indirectly in |InvalidateObservers...|.
 
   mHasPendingInvalidation = false;
-  SurfaceCache::RemoveImage(ImageKey(this));
 
-  if (UpdateImageContainer(Nothing())) {
-    // If we have image containers, that means we probably won't get a Draw call
-    // from the owner since they are using the container. We must assume all
-    // invalidations need to be handled.
+  if (SurfaceCache::InvalidateImage(ImageKey(this))) {
+    // If we still have recordings in the cache, make sure we handle future
+    // invalidations.
     MOZ_ASSERT(mRenderingObserver, "Should have a rendering observer by now");
     mRenderingObserver->ResumeHonoringInvalidations();
   }
@@ -621,10 +612,10 @@ VectorImage::GetType(uint16_t* aType) {
 
 //******************************************************************************
 NS_IMETHODIMP
-VectorImage::GetProducerId(uint32_t* aId) {
+VectorImage::GetProviderId(uint32_t* aId) {
   NS_ENSURE_ARG_POINTER(aId);
 
-  *aId = ImageResource::GetImageProducerId();
+  *aId = ImageResource::GetImageProviderId();
   return NS_OK;
 }
 
@@ -687,132 +678,66 @@ VectorImage::GetFrame(uint32_t aWhichFrame, uint32_t aFlags) {
 NS_IMETHODIMP_(already_AddRefed<SourceSurface>)
 VectorImage::GetFrameAtSize(const IntSize& aSize, uint32_t aWhichFrame,
                             uint32_t aFlags) {
+  MOZ_ASSERT(aWhichFrame <= FRAME_MAX_VALUE);
+
   AutoProfilerImagePaintMarker PROFILER_RAII(this);
 #ifdef DEBUG
   NotifyDrawingObservers();
 #endif
 
-  auto result =
-      GetFrameInternal(aSize, Nothing(), Nothing(), aWhichFrame, aFlags);
-  return Get<2>(result).forget();
-}
-
-Tuple<ImgDrawResult, IntSize, RefPtr<SourceSurface>>
-VectorImage::GetFrameInternal(const IntSize& aSize,
-                              const Maybe<SVGImageContext>& aSVGContext,
-                              const Maybe<ImageIntRegion>& aRegion,
-                              uint32_t aWhichFrame, uint32_t aFlags) {
-  MOZ_ASSERT(aWhichFrame <= FRAME_MAX_VALUE);
-
-  if (aSize.IsEmpty() || aWhichFrame > FRAME_MAX_VALUE) {
-    return MakeTuple(ImgDrawResult::BAD_ARGS, aSize, RefPtr<SourceSurface>());
-  }
-
-  if (mError) {
-    return MakeTuple(ImgDrawResult::BAD_IMAGE, aSize, RefPtr<SourceSurface>());
-  }
-
-  if (!mIsFullyLoaded) {
-    return MakeTuple(ImgDrawResult::NOT_READY, aSize, RefPtr<SourceSurface>());
+  if (aSize.IsEmpty() || aWhichFrame > FRAME_MAX_VALUE || mError ||
+      !mIsFullyLoaded) {
+    return nullptr;
   }
 
   uint32_t whichFrame = mHaveAnimations ? aWhichFrame : FRAME_FIRST;
 
-  RefPtr<SourceSurface> sourceSurface;
-  IntSize decodeSize;
-  Tie(sourceSurface, decodeSize) =
-      LookupCachedSurface(aSize, aSVGContext, aFlags);
+  auto [sourceSurface, decodeSize] =
+      LookupCachedSurface(aSize, SVGImageContext(), aFlags);
   if (sourceSurface) {
-    return MakeTuple(ImgDrawResult::SUCCESS, decodeSize,
-                     std::move(sourceSurface));
+    return sourceSurface.forget();
   }
 
   if (mSVGDocumentWrapper->IsDrawing()) {
     NS_WARNING("Refusing to make re-entrant call to VectorImage::Draw");
-    return MakeTuple(ImgDrawResult::TEMPORARY_ERROR, decodeSize,
-                     RefPtr<SourceSurface>());
+    return nullptr;
   }
 
   float animTime = (whichFrame == FRAME_FIRST)
                        ? 0.0f
                        : mSVGDocumentWrapper->GetCurrentTimeAsFloat();
 
-  // If we aren't given a region, create one that covers the whole SVG image.
-  ImageRegion region =
-      aRegion ? aRegion->ToImageRegion() : ImageRegion::Create(decodeSize);
-
   // By using a null gfxContext, we ensure that we will always attempt to
   // create a surface, even if we aren't capable of caching it (e.g. due to our
   // flags, having an animation, etc). Otherwise CreateSurface will assume that
   // the caller is capable of drawing directly to its own draw target if we
   // cannot cache.
-  SVGDrawingParameters params(nullptr, decodeSize, aSize, region,
-                              SamplingFilter::POINT, aSVGContext, animTime,
-                              aFlags, 1.0);
-
-  // Blob recorded vector images just create a simple surface responsible for
-  // generating blob keys and recording bindings. The recording won't happen
-  // until the caller requests the key after GetImageContainerAtSize.
-  if (aFlags & FLAG_RECORD_BLOB) {
-    RefPtr<SourceSurface> surface =
-        new SourceSurfaceBlobImage(mSVGDocumentWrapper, aSVGContext, aRegion,
-                                   decodeSize, whichFrame, aFlags);
-
-    return MakeTuple(ImgDrawResult::SUCCESS, decodeSize, std::move(surface));
-  }
+  SVGImageContext svgContext;
+  SVGDrawingParameters params(
+      nullptr, decodeSize, aSize, ImageRegion::Create(decodeSize),
+      SamplingFilter::POINT, svgContext, animTime, aFlags, 1.0);
 
   bool didCache;  // Was the surface put into the cache?
-  bool contextPaint = aSVGContext && aSVGContext->GetContextPaint();
 
-  AutoRestoreSVGState autoRestore(params, mSVGDocumentWrapper, contextPaint);
+  AutoRestoreSVGState autoRestore(params, mSVGDocumentWrapper,
+                                  /* aContextPaint */ false);
 
   RefPtr<gfxDrawable> svgDrawable = CreateSVGDrawable(params);
   RefPtr<SourceSurface> surface = CreateSurface(params, svgDrawable, didCache);
   if (!surface) {
     MOZ_ASSERT(!didCache);
-    return MakeTuple(ImgDrawResult::TEMPORARY_ERROR, decodeSize,
-                     RefPtr<SourceSurface>());
+    return nullptr;
   }
 
   SendFrameComplete(didCache, params.flags);
-  return MakeTuple(ImgDrawResult::SUCCESS, decodeSize, std::move(surface));
-}
-
-//******************************************************************************
-Tuple<ImgDrawResult, IntSize> VectorImage::GetImageContainerSize(
-    LayerManager* aManager, const IntSize& aSize, uint32_t aFlags) {
-  if (mError) {
-    return MakeTuple(ImgDrawResult::BAD_IMAGE, IntSize(0, 0));
-  }
-
-  if (!mIsFullyLoaded) {
-    return MakeTuple(ImgDrawResult::NOT_READY, IntSize(0, 0));
-  }
-
-  if (mHaveAnimations && !StaticPrefs::image_svg_blob_image()) {
-    // We don't support rasterizing animation SVGs. We can put them in a blob
-    // recording however instead of using fallback.
-    return MakeTuple(ImgDrawResult::NOT_SUPPORTED, IntSize(0, 0));
-  }
-
-  if (aManager->GetBackendType() != LayersBackend::LAYERS_WR) {
-    return MakeTuple(ImgDrawResult::NOT_SUPPORTED, IntSize(0, 0));
-  }
-
-  // We don't need to check if the size is too big since we only support
-  // WebRender backends.
-  if (aSize.IsEmpty()) {
-    return MakeTuple(ImgDrawResult::BAD_ARGS, IntSize(0, 0));
-  }
-
-  return MakeTuple(ImgDrawResult::SUCCESS, aSize);
+  return surface.forget();
 }
 
 NS_IMETHODIMP_(bool)
-VectorImage::IsImageContainerAvailable(LayerManager* aManager,
+VectorImage::IsImageContainerAvailable(WindowRenderer* aRenderer,
                                        uint32_t aFlags) {
   if (mError || !mIsFullyLoaded ||
-      aManager->GetBackendType() != LayersBackend::LAYERS_WR) {
+      aRenderer->GetBackendType() != LayersBackend::LAYERS_WR) {
     return false;
   }
 
@@ -825,86 +750,201 @@ VectorImage::IsImageContainerAvailable(LayerManager* aManager,
   return true;
 }
 
-//******************************************************************************
-NS_IMETHODIMP_(already_AddRefed<ImageContainer>)
-VectorImage::GetImageContainer(LayerManager* aManager, uint32_t aFlags) {
-  MOZ_ASSERT(aManager->GetBackendType() != LayersBackend::LAYERS_WR,
-             "WebRender should always use GetImageContainerAvailableAtSize!");
-  return nullptr;
-}
-
-//******************************************************************************
-NS_IMETHODIMP_(bool)
-VectorImage::IsImageContainerAvailableAtSize(LayerManager* aManager,
-                                             const IntSize& aSize,
-                                             uint32_t aFlags) {
-  // Since we only support image containers with WebRender, and it can handle
-  // textures larger than the hw max texture size, we don't need to check aSize.
-  return !aSize.IsEmpty() && IsImageContainerAvailable(aManager, aFlags);
-}
-
 //******************************************************************************
 NS_IMETHODIMP_(ImgDrawResult)
-VectorImage::GetImageContainerAtSize(layers::LayerManager* aManager,
-                                     const gfx::IntSize& aSize,
-                                     const Maybe<SVGImageContext>& aSVGContext,
-                                     const Maybe<ImageIntRegion>& aRegion,
-                                     uint32_t aFlags,
-                                     layers::ImageContainer** aOutContainer) {
-  Maybe<SVGImageContext> newSVGContext;
-  MaybeRestrictSVGContext(newSVGContext, aSVGContext, aFlags);
+VectorImage::GetImageProvider(WindowRenderer* aRenderer,
+                              const gfx::IntSize& aSize,
+                              const SVGImageContext& aSVGContext,
+                              const Maybe<ImageIntRegion>& aRegion,
+                              uint32_t aFlags,
+                              WebRenderImageProvider** aProvider) {
+  MOZ_ASSERT(NS_IsMainThread());
+  MOZ_ASSERT(aRenderer);
+  MOZ_ASSERT(!(aFlags & FLAG_BYPASS_SURFACE_CACHE), "Unsupported flags");
 
-  // The aspect ratio flag was already handled as part of the SVG context
-  // restriction above.
-  uint32_t flags = aFlags & ~(FLAG_FORCE_PRESERVEASPECTRATIO_NONE);
-  auto rv = GetImageContainerImpl(aManager, aSize,
-                                  newSVGContext ? newSVGContext : aSVGContext,
-                                  aRegion, flags, aOutContainer);
-
-  // Invalidations may still be suspended if we had a refresh tick and there
-  // were no image containers remaining. If we created a new container, we
-  // should resume invalidations to allow animations to progress.
-  if (*aOutContainer && mRenderingObserver) {
-    mRenderingObserver->ResumeHonoringInvalidations();
+  // We don't need to check if the size is too big since we only support
+  // WebRender backends.
+  if (aSize.IsEmpty()) {
+    return ImgDrawResult::BAD_ARGS;
   }
 
-  return rv;
+  if (mError) {
+    return ImgDrawResult::BAD_IMAGE;
+  }
+
+  if (!mIsFullyLoaded) {
+    return ImgDrawResult::NOT_READY;
+  }
+
+  if (mHaveAnimations && !(aFlags & FLAG_RECORD_BLOB)) {
+    // We don't support rasterizing animation SVGs. We can put them in a blob
+    // recording however instead of using fallback.
+    return ImgDrawResult::NOT_SUPPORTED;
+  }
+
+  AutoProfilerImagePaintMarker PROFILER_RAII(this);
+#ifdef DEBUG
+  NotifyDrawingObservers();
+#endif
+
+  // Only blob recordings support a region to restrict drawing.
+  const bool blobRecording = aFlags & FLAG_RECORD_BLOB;
+  MOZ_ASSERT_IF(!blobRecording, aRegion.isNothing());
+
+  LookupResult result(MatchType::NOT_FOUND);
+  auto playbackType =
+      mHaveAnimations ? PlaybackType::eAnimated : PlaybackType::eStatic;
+  auto surfaceFlags = ToSurfaceFlags(aFlags);
+
+  SurfaceKey surfaceKey =
+      VectorSurfaceKey(aSize, aRegion, aSVGContext, surfaceFlags, playbackType);
+  if ((aFlags & FLAG_SYNC_DECODE) || !(aFlags & FLAG_HIGH_QUALITY_SCALING)) {
+    result = SurfaceCache::Lookup(ImageKey(this), surfaceKey,
+                                  /* aMarkUsed = */ true);
+  } else {
+    result = SurfaceCache::LookupBestMatch(ImageKey(this), surfaceKey,
+                                           /* aMarkUsed = */ true);
+  }
+
+  // Unless we get a best match (exact or factor of 2 limited), then we want to
+  // generate a new recording/rerasterize, even if we have a substitute.
+  if (result && (result.Type() == MatchType::EXACT ||
+                 result.Type() == MatchType::SUBSTITUTE_BECAUSE_BEST)) {
+    result.Surface().TakeProvider(aProvider);
+    return ImgDrawResult::SUCCESS;
+  }
+
+  // Ensure we store the surface with the correct key if we switched to factor
+  // of 2 sizing or we otherwise got clamped.
+  IntSize rasterSize(aSize);
+  if (!result.SuggestedSize().IsEmpty()) {
+    rasterSize = result.SuggestedSize();
+    surfaceKey = surfaceKey.CloneWithSize(rasterSize);
+  }
+
+  // We're about to rerasterize, which may mean that some of the previous
+  // surfaces we've rasterized aren't useful anymore. We can allow them to
+  // expire from the cache by unlocking them here, and then sending out an
+  // invalidation. If this image is locked, any surfaces that are still useful
+  // will become locked again when Draw touches them, and the remainder will
+  // eventually expire.
+  bool mayCache = SurfaceCache::CanHold(rasterSize);
+  if (mayCache) {
+    SurfaceCache::UnlockEntries(ImageKey(this));
+  }
+
+  // Blob recorded vector images just create a provider responsible for
+  // generating blob keys and recording bindings. The recording won't happen
+  // until the caller requests the key explicitly.
+  RefPtr<ISurfaceProvider> provider;
+  if (blobRecording) {
+    provider = MakeRefPtr<BlobSurfaceProvider>(ImageKey(this), surfaceKey,
+                                               mSVGDocumentWrapper, aFlags);
+  } else {
+    if (mSVGDocumentWrapper->IsDrawing()) {
+      NS_WARNING("Refusing to make re-entrant call to VectorImage::Draw");
+      return ImgDrawResult::TEMPORARY_ERROR;
+    }
+
+    if (!SurfaceCache::IsLegalSize(rasterSize) ||
+        !Factory::AllowedSurfaceSize(rasterSize)) {
+      // If either of these is true then the InitWithDrawable call below will
+      // fail, so fail early and use this opportunity to return NOT_SUPPORTED
+      // instead of TEMPORARY_ERROR as we do for any InitWithDrawable failure.
+      // This means that we will use fallback which has a path that will draw
+      // directly into the gfxContext without having to allocate a surface. It
+      // means we will have to use fallback and re-rasterize for everytime we
+      // have to draw this image, but it's better than not drawing anything at
+      // all.
+      return ImgDrawResult::NOT_SUPPORTED;
+    }
+
+    // We aren't using blobs, so we need to rasterize.
+    float animTime =
+        mHaveAnimations ? mSVGDocumentWrapper->GetCurrentTimeAsFloat() : 0.0f;
+
+    // By using a null gfxContext, we ensure that we will always attempt to
+    // create a surface, even if we aren't capable of caching it (e.g. due to
+    // our flags, having an animation, etc). Otherwise CreateSurface will assume
+    // that the caller is capable of drawing directly to its own draw target if
+    // we cannot cache.
+    SVGDrawingParameters params(
+        nullptr, rasterSize, aSize, ImageRegion::Create(rasterSize),
+        SamplingFilter::POINT, aSVGContext, animTime, aFlags, 1.0);
+
+    RefPtr<gfxDrawable> svgDrawable = CreateSVGDrawable(params);
+    bool contextPaint = aSVGContext.GetContextPaint();
+    AutoRestoreSVGState autoRestore(params, mSVGDocumentWrapper, contextPaint);
+
+    mSVGDocumentWrapper->UpdateViewportBounds(params.viewportSize);
+    mSVGDocumentWrapper->FlushImageTransformInvalidation();
+
+    // Given we have no context, the default backend is fine.
+    BackendType backend =
+        gfxPlatform::GetPlatform()->GetDefaultContentBackend();
+
+    // Try to create an imgFrame, initializing the surface it contains by
+    // drawing our gfxDrawable into it. (We use FILTER_NEAREST since we never
+    // scale here.)
+    auto frame = MakeNotNull<RefPtr<imgFrame>>();
+    nsresult rv = frame->InitWithDrawable(
+        svgDrawable, params.size, SurfaceFormat::OS_RGBA, SamplingFilter::POINT,
+        params.flags, backend);
+
+    // If we couldn't create the frame, it was probably because it would end
+    // up way too big. Generally it also wouldn't fit in the cache, but the
+    // prefs could be set such that the cache isn't the limiting factor.
+    if (NS_FAILED(rv)) {
+      return ImgDrawResult::TEMPORARY_ERROR;
+    }
+
+    provider =
+        MakeRefPtr<SimpleSurfaceProvider>(ImageKey(this), surfaceKey, frame);
+  }
+
+  if (mayCache) {
+    // Attempt to cache the frame.
+    if (SurfaceCache::Insert(WrapNotNull(provider)) == InsertOutcome::SUCCESS) {
+      if (rasterSize != aSize) {
+        // We created a new surface that wasn't the size we requested, which
+        // means we entered factor-of-2 mode. We should purge any surfaces we
+        // no longer need rather than waiting for the cache to expire them.
+        SurfaceCache::PruneImage(ImageKey(this));
+      }
+
+      SendFrameComplete(/* aDidCache */ true, aFlags);
+    }
+  }
+
+  MOZ_ASSERT(provider);
+  provider.forget(aProvider);
+  return ImgDrawResult::SUCCESS;
 }
 
-bool VectorImage::MaybeRestrictSVGContext(
-    Maybe<SVGImageContext>& aNewSVGContext,
-    const Maybe<SVGImageContext>& aSVGContext, uint32_t aFlags) {
-  bool overridePAR =
-      (aFlags & FLAG_FORCE_PRESERVEASPECTRATIO_NONE) && aSVGContext;
+bool VectorImage::MaybeRestrictSVGContext(SVGImageContext& aSVGContext,
+                                          uint32_t aFlags) {
+  bool overridePAR = (aFlags & FLAG_FORCE_PRESERVEASPECTRATIO_NONE);
 
-  bool haveContextPaint = aSVGContext && aSVGContext->GetContextPaint();
-  bool blockContextPaint = false;
-  if (haveContextPaint) {
-    blockContextPaint = !SVGContextPaint::IsAllowedForImageFromURI(mURI);
-  }
+  bool haveContextPaint = aSVGContext.GetContextPaint();
+  bool blockContextPaint =
+      haveContextPaint && !SVGContextPaint::IsAllowedForImageFromURI(mURI);
 
   if (overridePAR || blockContextPaint) {
-    // The key that we create for the image surface cache must match the way
-    // that the image will be painted, so we need to initialize a new matching
-    // SVGImageContext here in order to generate the correct key.
-
-    aNewSVGContext = aSVGContext;  // copy
-
     if (overridePAR) {
       // The SVGImageContext must take account of the preserveAspectRatio
       // override:
-      MOZ_ASSERT(!aSVGContext->GetPreserveAspectRatio(),
+      MOZ_ASSERT(!aSVGContext.GetPreserveAspectRatio(),
                  "FLAG_FORCE_PRESERVEASPECTRATIO_NONE is not expected if a "
                  "preserveAspectRatio override is supplied");
       Maybe<SVGPreserveAspectRatio> aspectRatio = Some(SVGPreserveAspectRatio(
           SVG_PRESERVEASPECTRATIO_NONE, SVG_MEETORSLICE_UNKNOWN));
-      aNewSVGContext->SetPreserveAspectRatio(aspectRatio);
+      aSVGContext.SetPreserveAspectRatio(aspectRatio);
     }
 
     if (blockContextPaint) {
       // The SVGImageContext must not include context paint if the image is
       // not allowed to use it:
-      aNewSVGContext->ClearContextPaint();
+      aSVGContext.ClearContextPaint();
     }
   }
 
@@ -916,7 +956,7 @@ NS_IMETHODIMP_(ImgDrawResult)
 VectorImage::Draw(gfxContext* aContext, const nsIntSize& aSize,
                   const ImageRegion& aRegion, uint32_t aWhichFrame,
                   SamplingFilter aSamplingFilter,
-                  const Maybe<SVGImageContext>& aSVGContext, uint32_t aFlags,
+                  const SVGImageContext& aSVGContext, uint32_t aFlags,
                   float aOpacity) {
   if (aWhichFrame > FRAME_MAX_VALUE) {
     return ImgDrawResult::BAD_ARGS;
@@ -934,7 +974,7 @@ VectorImage::Draw(gfxContext* aContext, const nsIntSize& aSize,
     return ImgDrawResult::NOT_READY;
   }
 
-  if (mAnimationConsumers == 0) {
+  if (mAnimationConsumers == 0 && mHaveAnimations) {
     SendOnUnlockedDraw(aFlags);
   }
 
@@ -942,18 +982,12 @@ VectorImage::Draw(gfxContext* aContext, const nsIntSize& aSize,
   // - We are using a DrawTargetRecording because we prefer the drawing commands
   //   in general to the rasterized surface. This allows blob images to avoid
   //   rasterized SVGs with WebRender.
-  // - The size exceeds what we are willing to cache as a rasterized surface.
-  //   We don't do this for WebRender because the performance of the fallback
-  //   path is quite bad and upscaling the SVG from the clamped size is better
-  //   than bringing the browser to a crawl.
-  if (aContext->GetDrawTarget()->GetBackendType() == BackendType::RECORDING ||
-      (!gfxVars::UseWebRender() &&
-       aSize != SurfaceCache::ClampVectorSize(aSize))) {
+  if (aContext->GetDrawTarget()->GetBackendType() == BackendType::RECORDING) {
     aFlags |= FLAG_BYPASS_SURFACE_CACHE;
   }
 
   MOZ_ASSERT(!(aFlags & FLAG_FORCE_PRESERVEASPECTRATIO_NONE) ||
-                 (aSVGContext && aSVGContext->GetViewportSize()),
+                 aSVGContext.GetViewportSize(),
              "Viewport size is required when using "
              "FLAG_FORCE_PRESERVEASPECTRATIO_NONE");
 
@@ -963,18 +997,16 @@ VectorImage::Draw(gfxContext* aContext, const nsIntSize& aSize,
                        ? 0.0f
                        : mSVGDocumentWrapper->GetCurrentTimeAsFloat();
 
-  Maybe<SVGImageContext> newSVGContext;
-  bool contextPaint =
-      MaybeRestrictSVGContext(newSVGContext, aSVGContext, aFlags);
+  SVGImageContext newSVGContext = aSVGContext;
+  bool contextPaint = MaybeRestrictSVGContext(newSVGContext, aFlags);
 
   SVGDrawingParameters params(aContext, aSize, aSize, aRegion, aSamplingFilter,
-                              newSVGContext ? newSVGContext : aSVGContext,
-                              animTime, aFlags, aOpacity);
+                              newSVGContext, animTime, aFlags, aOpacity);
 
   // If we have an prerasterized version of this image that matches the
   // drawing parameters, use that.
   RefPtr<SourceSurface> sourceSurface;
-  Tie(sourceSurface, params.size) =
+  std::tie(sourceSurface, params.size) =
       LookupCachedSurface(aSize, params.svgContext, aFlags);
   if (sourceSurface) {
     RefPtr<gfxDrawable> drawable =
@@ -1017,16 +1049,15 @@ already_AddRefed<gfxDrawable> VectorImage::CreateSVGDrawable(
   return svgDrawable.forget();
 }
 
-Tuple<RefPtr<SourceSurface>, IntSize> VectorImage::LookupCachedSurface(
-    const IntSize& aSize, const Maybe<SVGImageContext>& aSVGContext,
-    uint32_t aFlags) {
+std::tuple<RefPtr<SourceSurface>, IntSize> VectorImage::LookupCachedSurface(
+    const IntSize& aSize, const SVGImageContext& aSVGContext, uint32_t aFlags) {
   // We can't use cached surfaces if we:
   // - Explicitly disallow it via FLAG_BYPASS_SURFACE_CACHE
   // - Want a blob recording which aren't supported by the cache.
   // - Have animations which aren't supported by the cache.
   if (aFlags & (FLAG_BYPASS_SURFACE_CACHE | FLAG_RECORD_BLOB) ||
       mHaveAnimations) {
-    return MakeTuple(RefPtr<SourceSurface>(), aSize);
+    return std::make_tuple(RefPtr<SourceSurface>(), aSize);
   }
 
   LookupResult result(MatchType::NOT_FOUND);
@@ -1044,7 +1075,7 @@ Tuple<RefPtr<SourceSurface>, IntSize> VectorImage::LookupCachedSurface(
   MOZ_ASSERT(result.Type() != MatchType::SUBSTITUTE_BECAUSE_PENDING);
   if (!result || result.Type() == MatchType::SUBSTITUTE_BECAUSE_NOT_FOUND) {
     // No matching surface, or the OS freed the volatile buffer.
-    return MakeTuple(RefPtr<SourceSurface>(), rasterSize);
+    return std::make_tuple(RefPtr<SourceSurface>(), rasterSize);
   }
 
   RefPtr<SourceSurface> sourceSurface = result.Surface()->GetSourceSurface();
@@ -1052,10 +1083,10 @@ Tuple<RefPtr<SourceSurface>, IntSize> VectorImage::LookupCachedSurface(
     // Something went wrong. (Probably a GPU driver crash or device reset.)
     // Attempt to recover.
     RecoverFromLossOfSurfaces();
-    return MakeTuple(RefPtr<SourceSurface>(), rasterSize);
+    return std::make_tuple(RefPtr<SourceSurface>(), rasterSize);
   }
 
-  return MakeTuple(std::move(sourceSurface), rasterSize);
+  return std::make_tuple(std::move(sourceSurface), rasterSize);
 }
 
 already_AddRefed<SourceSurface> VectorImage::CreateSurface(
@@ -1099,6 +1130,12 @@ already_AddRefed<SourceSurface> VectorImage::CreateSurface(
       aParams.context ? aParams.context->GetDrawTarget()->GetBackendType()
                       : gfxPlatform::GetPlatform()->GetDefaultContentBackend();
 
+  if (backend == BackendType::DIRECT2D1_1) {
+    // We don't want to draw arbitrary content with D2D anymore
+    // because it doesn't support PushLayerWithBlend so switch to skia
+    backend = BackendType::SKIA;
+  }
+
   // Try to create an imgFrame, initializing the surface it contains by drawing
   // our gfxDrawable into it. (We use FILTER_NEAREST since we never scale here.)
   auto frame = MakeNotNull<RefPtr<imgFrame>>();
@@ -1160,7 +1197,7 @@ void VectorImage::SendFrameComplete(bool aDidCache, uint32_t aFlags) {
                                          GetMaxSizedIntRect());
   } else {
     NotNull<RefPtr<VectorImage>> image = WrapNotNull(this);
-    NS_DispatchToMainThread(CreateMediumHighRunnable(NS_NewRunnableFunction(
+    NS_DispatchToMainThread(CreateRenderBlockingRunnable(NS_NewRunnableFunction(
         "ProgressTracker::SyncNotifyProgress", [=]() -> void {
           RefPtr<ProgressTracker> tracker = image->GetProgressTracker();
           if (tracker) {
@@ -1178,10 +1215,11 @@ void VectorImage::Show(gfxDrawable* aDrawable,
   gfxContextMatrixAutoSaveRestore saveMatrix(aParams.context);
   ImageRegion region(aParams.region);
   if (aParams.drawSize != aParams.size) {
-    gfx::Size scale(double(aParams.drawSize.width) / aParams.size.width,
-                    double(aParams.drawSize.height) / aParams.size.height);
-    aParams.context->Multiply(gfxMatrix::Scaling(scale.width, scale.height));
-    region.Scale(1.0 / scale.width, 1.0 / scale.height);
+    gfx::MatrixScales scale(
+        double(aParams.drawSize.width) / aParams.size.width,
+        double(aParams.drawSize.height) / aParams.size.height);
+    aParams.context->Multiply(gfx::Matrix::Scaling(scale));
+    region.Scale(1.0 / scale.xScale, 1.0 / scale.yScale);
   }
 
   MOZ_ASSERT(aDrawable, "Should have a gfxDrawable by now");
@@ -1218,6 +1256,11 @@ bool VectorImage::StartDecodingWithResult(uint32_t aFlags,
   return mIsFullyLoaded;
 }
 
+bool VectorImage::HasDecodedPixels() {
+  MOZ_ASSERT_UNREACHABLE("calling VectorImage::HasDecodedPixels");
+  return mIsFullyLoaded;
+}
+
 imgIContainer::DecodeResult VectorImage::RequestDecodeWithResult(
     uint32_t aFlags, uint32_t aWhichFrame) {
   // SVG images are ready to draw when they are loaded and don't have an error.
@@ -1523,7 +1566,7 @@ void VectorImage::InvalidateObserversOnNextRefreshDriverTick() {
   nsCOMPtr<nsIRunnable> ev(NS_NewRunnableFunction(
       "VectorImage::SendInvalidationNotifications",
       [=]() -> void { self->SendInvalidationNotifications(); }));
-  eventTarget->Dispatch(CreateMediumHighRunnable(ev.forget()),
+  eventTarget->Dispatch(CreateRenderBlockingRunnable(ev.forget()),
                         NS_DISPATCH_NORMAL);
 }
 
