@@ -10,12 +10,12 @@
 
 #include <stdlib.h>
 
-#include "mozilla/EndianUtils.h"
-#include "mozilla/Move.h"
-
-#include "mozilla/gfx/Swizzle.h"
+#include <utility>
 
 #include "RasterImage.h"
+#include "mozilla/EndianUtils.h"
+#include "mozilla/gfx/Swizzle.h"
+#include "mozilla/UniquePtrExtensions.h"
 
 using namespace mozilla::gfx;
 
@@ -155,7 +155,7 @@ LexerTransition<ICOState> nsICODecoder::ReadDirEntry(const char* aData) {
     e.mBitCount = LittleEndian::readUint16(aData + 6);
     e.mBytesInRes = LittleEndian::readUint32(aData + 8);
     e.mImageOffset = offset;
-    e.mSize = IntSize(e.mWidth, e.mHeight);
+    e.mSize = OrientedIntSize(e.mWidth, e.mHeight);
 
     // Only accept entries with sufficient resource data to actually contain
     // some image data.
@@ -235,7 +235,7 @@ LexerTransition<ICOState> nsICODecoder::FinishDirEntry() {
 
   // If an explicit output size was specified, we'll try to select the resource
   // that matches it best below.
-  const Maybe<IntSize> desiredSize = ExplicitOutputSize();
+  const Maybe<OrientedIntSize> desiredSize = ExplicitOutputSize();
 
   // Determine the biggest resource. We always use the biggest resource for the
   // intrinsic size, and if we don't have a specific desired size, we select it
@@ -299,12 +299,18 @@ LexerTransition<ICOState> nsICODecoder::FinishDirEntry() {
     return Transition::TerminateSuccess();
   }
 
-  // If the resource we selected matches the output size perfectly, we don't
-  // need to do any downscaling.
   if (mDirEntry->mSize == OutputSize()) {
+    // If the resource we selected matches the output size perfectly, we don't
+    // need to do any downscaling.
     MOZ_ASSERT_IF(desiredSize, mDirEntry->mSize == *desiredSize);
     MOZ_ASSERT_IF(!desiredSize, mDirEntry->mSize == Size());
-    mDownscaler.reset();
+  } else if (OutputSize().width < mDirEntry->mSize.width ||
+             OutputSize().height < mDirEntry->mSize.height) {
+    // Create a downscaler if we need to downscale.
+    //
+    // TODO(aosmond): This is the last user of Downscaler. We should switch this
+    // to SurfacePipe as well so we can remove the code from tree.
+    mDownscaler.emplace(OutputSize().ToUnknownSize());
   }
 
   size_t offsetToResource = mDirEntry->mImageOffset - FirstResourceOffset();
@@ -343,7 +349,7 @@ LexerTransition<ICOState> nsICODecoder::SniffResource(const char* aData) {
 
     // Create a PNG decoder which will do the rest of the work for us.
     bool metadataDecode = mReturnIterator.isSome();
-    Maybe<IntSize> expectedSize =
+    Maybe<OrientedIntSize> expectedSize =
         metadataDecode ? Nothing() : Some(mDirEntry->mSize);
     mContainedDecoder = DecoderFactory::CreateDecoderForICOResource(
         DecoderType::PNG, std::move(containedIterator.ref()), WrapNotNull(this),
@@ -353,16 +359,16 @@ LexerTransition<ICOState> nsICODecoder::SniffResource(const char* aData) {
     size_t toRead = mDirEntry->mBytesInRes - BITMAPINFOSIZE;
     return Transition::ToUnbuffered(ICOState::FINISHED_RESOURCE,
                                     ICOState::READ_RESOURCE, toRead);
-  } else {
-    // Make sure we have a sane size for the bitmap information header.
-    int32_t bihSize = LittleEndian::readUint32(aData);
-    if (bihSize != static_cast<int32_t>(BITMAPINFOSIZE)) {
-      return Transition::TerminateFailure();
-    }
-
-    // Read in the rest of the bitmap information header.
-    return ReadBIH(aData);
   }
+
+  // Make sure we have a sane size for the bitmap information header.
+  int32_t bihSize = LittleEndian::readUint32(aData);
+  if (bihSize != static_cast<int32_t>(BITMAPINFOSIZE)) {
+    return Transition::TerminateFailure();
+  }
+
+  // Read in the rest of the bitmap information header.
+  return ReadBIH(aData);
 }
 
 LexerTransition<ICOState> nsICODecoder::ReadResource() {
@@ -407,7 +413,7 @@ LexerTransition<ICOState> nsICODecoder::ReadBIH(const char* aData) {
   // Create a BMP decoder which will do most of the work for us; the exception
   // is the AND mask, which isn't present in standalone BMPs.
   bool metadataDecode = mReturnIterator.isSome();
-  Maybe<IntSize> expectedSize =
+  Maybe<OrientedIntSize> expectedSize =
       metadataDecode ? Nothing() : Some(mDirEntry->mSize);
   mContainedDecoder = DecoderFactory::CreateDecoderForICOResource(
       DecoderType::BMP, std::move(containedIterator.ref()), WrapNotNull(this),
@@ -485,11 +491,15 @@ LexerTransition<ICOState> nsICODecoder::PrepareForMask() {
     MOZ_ASSERT(bmpDecoder->GetImageDataLength() ==
                mDownscaler->TargetSize().width *
                    mDownscaler->TargetSize().height * sizeof(uint32_t));
-    mMaskBuffer = MakeUnique<uint8_t[]>(bmpDecoder->GetImageDataLength());
-    nsresult rv =
-        mDownscaler->BeginFrame(mDirEntry->mSize, Nothing(), mMaskBuffer.get(),
-                                /* aHasAlpha = */ true,
-                                /* aFlipVertically = */ true);
+    mMaskBuffer =
+        MakeUniqueFallible<uint8_t[]>(bmpDecoder->GetImageDataLength());
+    if (NS_WARN_IF(!mMaskBuffer)) {
+      return Transition::TerminateFailure();
+    }
+    nsresult rv = mDownscaler->BeginFrame(mDirEntry->mSize.ToUnknownSize(),
+                                          Nothing(), mMaskBuffer.get(),
+                                          /* aHasAlpha = */ true,
+                                          /* aFlipVertically = */ true);
     if (NS_FAILED(rv)) {
       return Transition::TerminateFailure();
     }
@@ -583,10 +593,10 @@ LexerTransition<ICOState> nsICODecoder::FinishMask() {
     }
     int32_t stride = mDownscaler->TargetSize().width * sizeof(uint32_t);
     DebugOnly<bool> ret =
-        // We know the format is B8G8R8A8 because we always assume bmp's inside
+        // We know the format is OS_RGBA because we always assume bmp's inside
         // ico's are transparent.
-        PremultiplyData(imageData, stride, SurfaceFormat::B8G8R8A8, imageData,
-                        stride, SurfaceFormat::B8G8R8A8,
+        PremultiplyData(imageData, stride, SurfaceFormat::OS_RGBA, imageData,
+                        stride, SurfaceFormat::OS_RGBA,
                         mDownscaler->TargetSize());
     MOZ_ASSERT(ret);
   }
