@@ -14,6 +14,7 @@ struct JSContext;
 #include "nsCOMPtr.h"
 #include "nsString.h"
 #include "mozilla/Attributes.h"
+#include "mozilla/gfx/PrintPromise.h"
 
 #include "nsCRT.h" /* should be <limits.h>? */
 
@@ -33,25 +34,24 @@ class nsDeviceContextSpecGTK : public nsIDeviceContextSpec {
 
   already_AddRefed<PrintTarget> MakePrintTarget() final;
 
-  NS_IMETHOD Init(nsIWidget* aWidget, nsIPrintSettings* aPS,
-                  bool aIsPrintPreview) override;
+  NS_IMETHOD Init(nsIPrintSettings* aPS, bool aIsPrintPreview) override;
   NS_IMETHOD BeginDocument(const nsAString& aTitle,
                            const nsAString& aPrintToFileName,
                            int32_t aStartPage, int32_t aEndPage) override;
-  NS_IMETHOD EndDocument() override;
+  RefPtr<mozilla::gfx::PrintEndDocumentPromise> EndDocument() override;
   NS_IMETHOD BeginPage() override { return NS_OK; }
   NS_IMETHOD EndPage() override { return NS_OK; }
 
  protected:
   virtual ~nsDeviceContextSpecGTK();
-  nsCOMPtr<nsPrintSettingsGTK> mPrintSettings;
-  bool mToPrinter : 1; /* If true, print to printer */
   GtkPrintSettings* mGtkPrintSettings;
   GtkPageSetup* mGtkPageSetup;
 
   nsCString mSpoolName;
   nsCOMPtr<nsIFile> mSpoolFile;
   nsCString mTitle;
+  // Helper for EnumeratePrinters / PrinterEnumerator:
+  bool mHasEnumerationFoundAMatch = false;
 
  private:
   void EnumeratePrinters();
