@@ -32,7 +32,8 @@ nsDeviceContextSpecProxy::nsDeviceContextSpecProxy(
 nsDeviceContextSpecProxy::~nsDeviceContextSpecProxy() = default;
 
 NS_IMETHODIMP
-nsDeviceContextSpecProxy::Init(nsIPrintSettings* aPrintSettings, bool aIsPrintPreview) {
+nsDeviceContextSpecProxy::Init(nsIPrintSettings* aPrintSettings,
+                               bool aIsPrintPreview) {
   mPrintSettings = aPrintSettings;
 
   if (aIsPrintPreview) {
@@ -114,28 +115,18 @@ nsDeviceContextSpecProxy::BeginDocument(const nsAString& aTitle,
   return rv;
 }
 
-NS_IMETHODIMP
+RefPtr<mozilla::gfx::PrintEndDocumentPromise>
 nsDeviceContextSpecProxy::EndDocument() {
   if (!mRemotePrintJob || mRemotePrintJob->IsDestroyed()) {
     mRemotePrintJob = nullptr;
-    return NS_ERROR_NOT_AVAILABLE;
+    return mozilla::gfx::PrintEndDocumentPromise::CreateAndReject(
+        NS_ERROR_NOT_AVAILABLE, __func__);
   }
 
   Unused << mRemotePrintJob->SendFinalizePrint();
 
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsDeviceContextSpecProxy::AbortDocument() {
-  if (!mRemotePrintJob || mRemotePrintJob->IsDestroyed()) {
-    mRemotePrintJob = nullptr;
-    return NS_ERROR_NOT_AVAILABLE;
-  }
-
-  Unused << mRemotePrintJob->SendAbortPrint(NS_OK);
-
-  return NS_OK;
+  return mozilla::gfx::PrintEndDocumentPromise::CreateAndResolve(true,
+                                                                 __func__);
 }
 
 NS_IMETHODIMP
