@@ -22,7 +22,6 @@
 #include "mozilla/Logging.h"
 #include "nsIClassInfoImpl.h"
 #include "nsAlgorithm.h"
-#include "nsMemory.h"
 #include "nsPipe.h"
 #include "nsIAsyncInputStream.h"
 #include "nsIAsyncOutputStream.h"
@@ -160,21 +159,21 @@ struct nsPipeReadState {
         mNeedDrain(false) {}
 
   // All members of this type are guarded by the pipe monitor, however it cannot
-  // be named from this type, so the less-reliable GUARDED_VAR is used instead.
-  // In the future it would be nice to avoid this, especially as GUARDED_VAR is
-  // deprecated.
-  char* mReadCursor GUARDED_VAR;
-  char* mReadLimit GUARDED_VAR;
-  int32_t mSegment GUARDED_VAR;
-  uint32_t mAvailable GUARDED_VAR;
+  // be named from this type, so the less-reliable MOZ_GUARDED_VAR is used
+  // instead. In the future it would be nice to avoid this, especially as
+  // MOZ_GUARDED_VAR is deprecated.
+  char* mReadCursor MOZ_GUARDED_VAR;
+  char* mReadLimit MOZ_GUARDED_VAR;
+  int32_t mSegment MOZ_GUARDED_VAR;
+  uint32_t mAvailable MOZ_GUARDED_VAR;
 
   // This flag is managed using the AutoReadSegment RAII stack class.
-  bool mActiveRead GUARDED_VAR;
+  bool mActiveRead MOZ_GUARDED_VAR;
 
   // Set to indicate that the input stream has closed and should be drained,
   // but that drain has been delayed due to an active read.  When the read
   // completes, this flag indicate the drain should then be performed.
-  bool mNeedDrain GUARDED_VAR;
+  bool mNeedDrain MOZ_GUARDED_VAR;
 };
 
 //-----------------------------------------------------------------------------
@@ -217,7 +216,7 @@ class nsPipeInputStream final : public nsIAsyncInputStream,
 
   void SetNonBlocking(bool aNonBlocking) { mBlocking = !aNonBlocking; }
 
-  uint32_t Available() REQUIRES(Monitor());
+  uint32_t Available() MOZ_REQUIRES(Monitor());
 
   // synchronously wait for the pipe to become readable.
   nsresult Wait();
@@ -227,10 +226,10 @@ class nsPipeInputStream final : public nsIAsyncInputStream,
   // evidence.
   MonitorAction OnInputReadable(uint32_t aBytesWritten, nsPipeEvents&,
                                 const ReentrantMonitorAutoEnter& ev)
-      REQUIRES(Monitor());
+      MOZ_REQUIRES(Monitor());
   MonitorAction OnInputException(nsresult, nsPipeEvents&,
                                  const ReentrantMonitorAutoEnter& ev)
-      REQUIRES(Monitor());
+      MOZ_REQUIRES(Monitor());
 
   nsPipeReadState& ReadState() { return mReadState; }
 
@@ -240,13 +239,13 @@ class nsPipeInputStream final : public nsIAsyncInputStream,
 
   // A version of Status() that doesn't acquire the monitor.
   nsresult Status(const ReentrantMonitorAutoEnter& ev) const
-      REQUIRES(Monitor());
+      MOZ_REQUIRES(Monitor());
 
   // The status of this input stream, ignoring the status of the underlying
   // monitor. If this status is errored, the input stream has either already
   // been removed from the pipe, or will be removed from the pipe shortly.
   nsresult InputStatus(const ReentrantMonitorAutoEnter&) const
-      REQUIRES(Monitor()) {
+      MOZ_REQUIRES(Monitor()) {
     return mInputStatus;
   }
 
@@ -261,12 +260,12 @@ class nsPipeInputStream final : public nsIAsyncInputStream,
   // Individual input streams can be closed without effecting the rest of the
   // pipe.  So track individual input stream status separately.  |mInputStatus|
   // is protected by |mPipe->mReentrantMonitor|.
-  nsresult mInputStatus GUARDED_BY(Monitor());
+  nsresult mInputStatus MOZ_GUARDED_BY(Monitor());
   bool mBlocking;
 
   // these variables can only be accessed while inside the pipe's monitor
-  bool mBlocked GUARDED_BY(Monitor());
-  CallbackHolder mCallback GUARDED_BY(Monitor());
+  bool mBlocked MOZ_GUARDED_BY(Monitor());
+  CallbackHolder mCallback MOZ_GUARDED_BY(Monitor());
 
   // requires pipe's monitor to access members; usually treat as an opaque token
   // to pass to nsPipe
@@ -299,15 +298,16 @@ class nsPipeOutputStream : public nsIAsyncOutputStream, public nsIClassInfo {
         mWritable(true) {}
 
   void SetNonBlocking(bool aNonBlocking) { mBlocking = !aNonBlocking; }
-  void SetWritable(bool aWritable) REQUIRES(Monitor()) {
+  void SetWritable(bool aWritable) MOZ_REQUIRES(Monitor()) {
     mWritable = aWritable;
   }
 
   // synchronously wait for the pipe to become writable.
   nsresult Wait();
 
-  MonitorAction OnOutputWritable(nsPipeEvents&) REQUIRES(Monitor());
-  MonitorAction OnOutputException(nsresult, nsPipeEvents&) REQUIRES(Monitor());
+  MonitorAction OnOutputWritable(nsPipeEvents&) MOZ_REQUIRES(Monitor());
+  MonitorAction OnOutputException(nsresult, nsPipeEvents&)
+      MOZ_REQUIRES(Monitor());
 
   ReentrantMonitor& Monitor() const;
 
@@ -320,9 +320,9 @@ class nsPipeOutputStream : public nsIAsyncOutputStream, public nsIClassInfo {
   bool mBlocking;
 
   // these variables can only be accessed while inside the pipe's monitor
-  bool mBlocked GUARDED_BY(Monitor());
-  bool mWritable GUARDED_BY(Monitor());
-  CallbackHolder mCallback GUARDED_BY(Monitor());
+  bool mBlocked MOZ_GUARDED_BY(Monitor());
+  bool mWritable MOZ_GUARDED_BY(Monitor());
+  CallbackHolder mCallback MOZ_GUARDED_BY(Monitor());
 };
 
 //-----------------------------------------------------------------------------
@@ -336,8 +336,8 @@ class nsPipe final {
   NS_INLINE_DECL_THREADSAFE_REFCOUNTING(nsPipe)
 
   // public constructor
-  friend nsresult NS_NewPipe2(nsIAsyncInputStream**, nsIAsyncOutputStream**,
-                              bool, bool, uint32_t, uint32_t);
+  friend void NS_NewPipe2(nsIAsyncInputStream**, nsIAsyncOutputStream**, bool,
+                          bool, uint32_t, uint32_t);
 
  private:
   nsPipe(uint32_t aSegmentSize, uint32_t aSegmentCount);
@@ -350,23 +350,26 @@ class nsPipe final {
   //
 
   void PeekSegment(const nsPipeReadState& aReadState, uint32_t aIndex,
-                   char*& aCursor, char*& aLimit) REQUIRES(mReentrantMonitor);
+                   char*& aCursor, char*& aLimit)
+      MOZ_REQUIRES(mReentrantMonitor);
   SegmentChangeResult AdvanceReadSegment(nsPipeReadState& aReadState,
                                          const ReentrantMonitorAutoEnter& ev)
-      REQUIRES(mReentrantMonitor);
+      MOZ_REQUIRES(mReentrantMonitor);
   bool ReadSegmentBeingWritten(nsPipeReadState& aReadState)
-      REQUIRES(mReentrantMonitor);
-  uint32_t CountSegmentReferences(int32_t aSegment) REQUIRES(mReentrantMonitor);
-  void SetAllNullReadCursors() REQUIRES(mReentrantMonitor);
-  bool AllReadCursorsMatchWriteCursor() REQUIRES(mReentrantMonitor);
-  void RollBackAllReadCursors(char* aWriteCursor) REQUIRES(mReentrantMonitor);
-  void UpdateAllReadCursors(char* aWriteCursor) REQUIRES(mReentrantMonitor);
-  void ValidateAllReadCursors() REQUIRES(mReentrantMonitor);
+      MOZ_REQUIRES(mReentrantMonitor);
+  uint32_t CountSegmentReferences(int32_t aSegment)
+      MOZ_REQUIRES(mReentrantMonitor);
+  void SetAllNullReadCursors() MOZ_REQUIRES(mReentrantMonitor);
+  bool AllReadCursorsMatchWriteCursor() MOZ_REQUIRES(mReentrantMonitor);
+  void RollBackAllReadCursors(char* aWriteCursor)
+      MOZ_REQUIRES(mReentrantMonitor);
+  void UpdateAllReadCursors(char* aWriteCursor) MOZ_REQUIRES(mReentrantMonitor);
+  void ValidateAllReadCursors() MOZ_REQUIRES(mReentrantMonitor);
   uint32_t GetBufferSegmentCount(const nsPipeReadState& aReadState,
                                  const ReentrantMonitorAutoEnter& ev) const
-      REQUIRES(mReentrantMonitor);
+      MOZ_REQUIRES(mReentrantMonitor);
   bool IsAdvanceBufferFull(const ReentrantMonitorAutoEnter& ev) const
-      REQUIRES(mReentrantMonitor);
+      MOZ_REQUIRES(mReentrantMonitor);
 
   //
   // methods below may be called while outside the pipe's monitor
@@ -397,39 +400,39 @@ class nsPipe final {
   // a weak reference as the streams will clear their entry here in their
   // destructor.  Using a strong reference would create a reference cycle.
   // Only usable while mReentrantMonitor is locked.
-  nsTArray<nsPipeInputStream*> mInputList GUARDED_BY(mReentrantMonitor);
+  nsTArray<nsPipeInputStream*> mInputList MOZ_GUARDED_BY(mReentrantMonitor);
 
   ReentrantMonitor mReentrantMonitor;
-  nsSegmentedBuffer mBuffer GUARDED_BY(mReentrantMonitor);
+  nsSegmentedBuffer mBuffer MOZ_GUARDED_BY(mReentrantMonitor);
 
   // The maximum number of segments to allow to be buffered in advance
   // of the fastest reader.  This is collection of segments is called
   // the "advance buffer".
-  uint32_t mMaxAdvanceBufferSegmentCount GUARDED_BY(mReentrantMonitor);
+  uint32_t mMaxAdvanceBufferSegmentCount MOZ_GUARDED_BY(mReentrantMonitor);
 
-  int32_t mWriteSegment GUARDED_BY(mReentrantMonitor);
-  char* mWriteCursor GUARDED_BY(mReentrantMonitor);
-  char* mWriteLimit GUARDED_BY(mReentrantMonitor);
+  int32_t mWriteSegment MOZ_GUARDED_BY(mReentrantMonitor);
+  char* mWriteCursor MOZ_GUARDED_BY(mReentrantMonitor);
+  char* mWriteLimit MOZ_GUARDED_BY(mReentrantMonitor);
 
   // |mStatus| is protected by |mReentrantMonitor|.
-  nsresult mStatus GUARDED_BY(mReentrantMonitor);
+  nsresult mStatus MOZ_GUARDED_BY(mReentrantMonitor);
 };
 
 //-----------------------------------------------------------------------------
 
 // Declarations of Monitor() methods on the streams.
 //
-// These must be placed early to provide RETURN_CAPABILITY annotations for the
-// thread-safety analysis. This couldn't be done at the declaration due to
+// These must be placed early to provide MOZ_RETURN_CAPABILITY annotations for
+// the thread-safety analysis. This couldn't be done at the declaration due to
 // nsPipe not yet being defined.
 
 ReentrantMonitor& nsPipeOutputStream::Monitor() const
-    RETURN_CAPABILITY(mPipe->mReentrantMonitor) {
+    MOZ_RETURN_CAPABILITY(mPipe->mReentrantMonitor) {
   return mPipe->mReentrantMonitor;
 }
 
 ReentrantMonitor& nsPipeInputStream::Monitor() const
-    RETURN_CAPABILITY(mPipe->mReentrantMonitor) {
+    MOZ_RETURN_CAPABILITY(mPipe->mReentrantMonitor) {
   return mPipe->mReentrantMonitor;
 }
 
@@ -571,7 +574,7 @@ nsPipe::nsPipe(uint32_t aSegmentSize, uint32_t aSegmentCount)
   // the size to expand when cloned streams are read at different
   // rates.  We enforce a limit on how much data can be buffered
   // ahead of the fastest reader in GetWriteSegment().
-  MOZ_ALWAYS_SUCCEEDS(mBuffer.Init(aSegmentSize, UINT32_MAX));
+  MOZ_ALWAYS_SUCCEEDS(mBuffer.Init(aSegmentSize));
 }
 
 nsPipe::~nsPipe() = default;
@@ -1313,6 +1316,12 @@ nsPipeInputStream::Available(uint64_t* aResult) {
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsPipeInputStream::StreamStatus() {
+  ReentrantMonitorAutoEnter mon(mPipe->mReentrantMonitor);
+  return mReadState.mAvailable ? NS_OK : Status(mon);
+}
+
 NS_IMETHODIMP
 nsPipeInputStream::ReadSegments(nsWriteSegmentFun aWriter, void* aClosure,
                                 uint32_t aCount, uint32_t* aReadCount) {
@@ -1716,11 +1725,17 @@ nsPipeOutputStream::Write(const char* aFromBuf, uint32_t aBufLen,
 }
 
 NS_IMETHODIMP
-nsPipeOutputStream::Flush(void) {
+nsPipeOutputStream::Flush() {
   // nothing to do
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsPipeOutputStream::StreamStatus() {
+  ReentrantMonitorAutoEnter mon(mPipe->mReentrantMonitor);
+  return mPipe->mStatus;
+}
+
 NS_IMETHODIMP
 nsPipeOutputStream::WriteFrom(nsIInputStream* aFromStream, uint32_t aCount,
                               uint32_t* aWriteCount) {
@@ -1767,9 +1782,9 @@ nsPipeOutputStream::AsyncWait(nsIOutputStreamCallback* aCallback,
 
 ////////////////////////////////////////////////////////////////////////////////
 
-nsresult NS_NewPipe(nsIInputStream** aPipeIn, nsIOutputStream** aPipeOut,
-                    uint32_t aSegmentSize, uint32_t aMaxSize,
-                    bool aNonBlockingInput, bool aNonBlockingOutput) {
+void NS_NewPipe(nsIInputStream** aPipeIn, nsIOutputStream** aPipeOut,
+                uint32_t aSegmentSize, uint32_t aMaxSize,
+                bool aNonBlockingInput, bool aNonBlockingOutput) {
   if (aSegmentSize == 0) {
     aSegmentSize = DEFAULT_SEGMENT_SIZE;
   }
@@ -1784,23 +1799,19 @@ nsresult NS_NewPipe(nsIInputStream** aPipeIn, nsIOutputStream** aPipeOut,
 
   nsIAsyncInputStream* in;
   nsIAsyncOutputStream* out;
-  nsresult rv = NS_NewPipe2(&in, &out, aNonBlockingInput, aNonBlockingOutput,
-                            aSegmentSize, segmentCount);
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
+  NS_NewPipe2(&in, &out, aNonBlockingInput, aNonBlockingOutput, aSegmentSize,
+              segmentCount);
 
   *aPipeIn = in;
   *aPipeOut = out;
-  return NS_OK;
 }
 
 // Disable thread safety analysis as this is logically a constructor, and no
 // additional threads can observe these objects yet.
-nsresult NS_NewPipe2(nsIAsyncInputStream** aPipeIn,
-                     nsIAsyncOutputStream** aPipeOut, bool aNonBlockingInput,
-                     bool aNonBlockingOutput, uint32_t aSegmentSize,
-                     uint32_t aSegmentCount) NO_THREAD_SAFETY_ANALYSIS {
+void NS_NewPipe2(nsIAsyncInputStream** aPipeIn, nsIAsyncOutputStream** aPipeOut,
+                 bool aNonBlockingInput, bool aNonBlockingOutput,
+                 uint32_t aSegmentSize,
+                 uint32_t aSegmentCount) MOZ_NO_THREAD_SAFETY_ANALYSIS {
   RefPtr<nsPipe> pipe =
       new nsPipe(aSegmentSize ? aSegmentSize : DEFAULT_SEGMENT_SIZE,
                  aSegmentCount ? aSegmentCount : DEFAULT_SEGMENT_COUNT);
@@ -1814,7 +1825,6 @@ nsresult NS_NewPipe2(nsIAsyncInputStream** aPipeIn,
 
   pipeIn.forget(aPipeIn);
   pipeOut.forget(aPipeOut);
-  return NS_OK;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -1841,9 +1851,10 @@ nsPipeHolder::Init(bool aNonBlockingInput, bool aNonBlockingOutput,
   if (mInput || mOutput) {
     return NS_ERROR_ALREADY_INITIALIZED;
   }
-  return NS_NewPipe2(getter_AddRefs(mInput), getter_AddRefs(mOutput),
-                     aNonBlockingInput, aNonBlockingOutput, aSegmentSize,
-                     aSegmentCount);
+  NS_NewPipe2(getter_AddRefs(mInput), getter_AddRefs(mOutput),
+              aNonBlockingInput, aNonBlockingOutput, aSegmentSize,
+              aSegmentCount);
+  return NS_OK;
 }
 
 NS_IMETHODIMP
